#!/usr/bin/python3
# Copyright (c) 2020 David Steele <dsteele@gmail.com>
#
# SPDX-License-Identifier: GPL-2.0-or-later
# License-Filename: LICENSE


import os
import sys
from pathlib import Path
from subprocess import run

import todo_txt_base.tdtlist as tdtlist
import todo_txt_base.tdtbackup as tdtbackup
from todo_txt_base.hooks import posthook, prehook, tdtwrapper, run_hooks


@tdtwrapper
@prehook
@posthook
def vi_todo(exe_path, task_path):
    run(["vi", task_path])


@tdtwrapper
@prehook
@posthook
def edit_todo(exe_path, task_path):
    run(["editor", task_path])


@tdtwrapper
def execute_todo(exe_path, task_path):
    phase = sys.argv[1]
    if phase in ["pre", "post"]:
        textfile = sys.argv[2]
        dir = Path("/etc/todo.txt-base/{}hooks/".format(phase))
        run_hooks(str(dir), task_path)


@tdtwrapper
@prehook
def list_todo(exe_path, task_path):
    sys.argv = ["listtodo", "-f", task_path] + sys.argv[1:]
    tdtlist.main()


@tdtwrapper
def backup_todo(exe_path, task_path):
    backup_path = Path(task_path).parent / "backup"
    sys.argv = ["backuptodo", "-f", task_path, "-b", str(backup_path)] + sys.argv[1:]
    tdtbackup.main()


@tdtwrapper
def cd_todo(exe_path, task_path):
    task_dir_path = Path(task_path).parent
    shell_path = os.getenv("SHELL")
    os.chdir(str(task_dir_path))
    run(shell_path)



#!/usr/bin/python3
# Copyright (c) 2020 David Steele <dsteele@gmail.com>
#
# SPDX-License-Identifier: GPL-2.0-or-later
# License-Filename: LICENSE

import argparse
import datetime
import functools
import json
import os
import pwd
import re
import shutil
import subprocess
import tempfile
import textwrap
import time
from pathlib import Path
from typing import List

from relatorio.templates.opendocument import Template

from todo_txt_base.utils import nullfd


def is_task(line: str, *terms: str) -> bool:
    if re.search(r"^\s*#", line):
        return False

    if "@" not in line:
        return False

    if re.search("^x ", line):
        return False

    if any(term not in line for term in terms):
        return False

    return True


def is_current_task(line: str, *terms: str) -> bool:
    if "@~" in line:
        return False

    if threshold_mask(line):
        return False

    return is_task(line, *terms)


def task_priority(task: str) -> str:
    match = re.search(r"^\((.)\) ", str(task))
    if match:
        return match.group(1)

    return "M"


def threshold_mask(task: str) -> bool:
    match = re.search(r"(^|[^\S])t:(\d\d\d\d-\d\d-\d\d)($|[^\S])", task)

    if not match:
        return False

    threshold_date = datetime.datetime.strptime(match.group(2), "%Y-%m-%d")

    return datetime.datetime.now() < threshold_date


def parse_args():
    parser = argparse.ArgumentParser(
        description="List the tasks in todo.txt, by @category",
        epilog=textwrap.dedent(
            """
            Process the todo.txt file, and save tasks lists, by context,
            in text and LibreOffice formats. The lists are saved in the
            same directory as tasks.txt and tasks.odt. Optionally, the
            LibreOffice list can be automatically opened.
            """[
                1:-1
            ]
        ),
    )

    parser.add_argument(
        "-f",
        "--file",
        help="the todo.txt file location "
        "(defaults to ~/Dropbox/todo/todo.txt)",
        default=os.path.expanduser("~/Dropbox/todo/todo.txt"),
    )

    parser.add_argument(
        "terms",
        nargs="*",
        metavar="TERM",
        help="search terms to filter the reported tasks",
    )

    parser.add_argument(
        "-p",
        "--priority",
        type=str,
        default="",
        help="Minimum priority to display",
    )

    parser.add_argument(
        "-t",
        "--text_output",
        action="store_true",
        help="Output to stdout",
    )

    args = parser.parse_args()

    return args


@functools.total_ordering
class tdline:
    def __init__(self, num: int, text: str) -> None:
        self.text = text
        self.num = num + 1

    def __str__(self) -> str:
        return self.text

    def _sort_key(self):
        """Sort tasks by (priority, tasknum, tasktext)."""
        return (task_priority(self.text), self.num, self.text)

    def __lt__(self, other: "tdline") -> bool:
        return self._sort_key() < other._sort_key()


def list_tasks(
    infile: str, outdir: str, terms: List[str], priority: str, textout: bool
):
    with open(infile, "r", encoding="utf-8") as fp:
        tdlines = [tdline(*x) for x in enumerate(fp.read().splitlines())]

    tasks = sorted([x for x in tdlines if is_current_task(str(x), *terms)])

    if priority:
        tasks = [x for x in tasks if task_priority(str(x)) <= priority]

    contexts = sorted(
        {y for x in tasks for y in str(x).split() if y[0] == "@"}
    )

    txt_file = os.path.join(outdir, "tasks.txt")
    odt_file = os.path.join(outdir, "tasks.odt")
    json_file = os.path.join(outdir, "tasks.json")
    template_path = Path(__file__).parent / "template.odt"

    proj_dict = {
        "date": str(datetime.datetime.now().strftime("%B %d, %Y")),
        "priority": priority,
        "priority_string": "",
        "terms": "",
        "contexts": [],
    }

    if priority:
        priority_string = "Priority {} and higher".format(priority)
        proj_dict["priority_string"] = priority_string

    if terms:
        term_list = ", ".join(['"' + x + '"' for x in terms])
        proj_dict["terms"] = term_list

    with open(txt_file, "w", encoding="utf-8") as txtfd:
        for context in contexts:
            context_dict = {"context": context, "tasks": []}
            proj_dict["contexts"].append(context_dict)

            txtfd.write("\n{}\n".format(context))

            for task in tasks:
                if context in str(task).split():
                    txtfd.write("{}\n".format(task))
                    context_dict["tasks"].append(
                        {"text": task.text, "num": task.num}
                    )

    with open(json_file, "w") as fp:
        json.dump(proj_dict, fp, indent=4)

    basic = Template(source="", filepath=template_path)
    basic_generated = basic.generate(o=proj_dict).render()
    with open(odt_file, "wb") as fp:
        fp.write(basic_generated.getvalue())

    if textout:
        txtPath = Path(txt_file)
        print(txtPath.read_text())

    else:
        with nullfd(1), nullfd(2):
            subprocess.Popen(
                ["xdg-open", odt_file],
                shell=False,
                stdin=None,
                stdout=None,
                stderr=None,
                close_fds=True,
            )


def tempdir():
    dirname = "tdtlist-{}".format(pwd.getpwuid(os.getuid())[0])
    tempdir = os.path.join(tempfile.gettempdir(), dirname)
    try:
        os.makedirs(tempdir)
    except:
        pass
    return tempdir


def trimtemp(tempdir, threshold=3600):
    subdirpath = [Path(x) for x in os.listdir(tempdir) if Path(x).is_dir()]

    tmpPath = Path(tempdir)
    for subdirPath in tmpPath.iterdir():
        age = time.time() - subdirPath.stat().st_mtime
        if subdirPath.is_dir() and age > threshold:
            shutil.rmtree(str(subdirPath.resolve()))


def main():
    args = parse_args()

    docdir = tempfile.mkdtemp(dir=tempdir())

    list_tasks(args.file, docdir, args.terms, args.priority, args.text_output)

    trimtemp(tempdir())


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import os
import sys
from pathlib import Path
from subprocess import run

# Set up paths
WORKDIR = "/tmp"
work_path = Path(WORKDIR)

# TODO - temp
if Path("/usr/bin/topydo-helper").exists():
    sys.exit(0)

if "AUTOPKGTEST_ARTIFACTS" in os.environ:
    artifact_path = Path(os.environ["AUTOPKGTEST_ARTIFACTS"])
else:
    artifact_path = work_path

config_path = work_path / "topydo.conf"
todo_path = artifact_path / "todo.txt"

start_path = Path(os.getcwd())
os.chdir(WORKDIR)

if config_path.exists():
    config_path.unlink()

if todo_path.exists():
    todo_path.unlink()

# Set up config
with open(str(config_path), "w") as fp:
    fp.write("[topydo]\n")
    fp.write("filename = %s\n" % str(todo_path))

# Can I add a tasks using topydo?
run("topydo add Foo".split(), capture_output=True, encoding="utf-8")
with open(str(todo_path), "r") as fp:
    todo_txt = fp.read()
assert "Foo" in todo_txt

# Can I list that task using todo.txt?
cp = run("todo.txt ls".split(), capture_output=True, encoding="utf-8")
assert "Foo" in cp.stdout

config_path.unlink()
todo_path.unlink()
(todo_path.parent / ".todo.bak").unlink()

os.chdir(str(start_path))



#!/bin/sh

set -e

if [ -z "$AUTOPKGTEST_ARTIFACTS" ] ; then
  BASEDIR=/tmp
else
  BASEDIR=$AUTOPKGTEST_ARTIFACTS
fi

TESTDIR="$BASEDIR"/backuptest
rm -rf "$TESTDIR"
mkdir "$TESTDIR"

TODO="$TESTDIR"/todo.txt
echo "Foo" > "$TODO"

BACKUPDIR="$TESTDIR/backup"
mkdir "$BACKUPDIR"

backuptodo -f "$TODO" -b "$BACKUPDIR"

cat "$BACKUPDIR"/* | grep -q "Foo"

echo "Test complete"



# Copyright (c) 2020 David Steele <dsteele@gmail.com>
#
# SPDX-License-Identifier: GPL-2.0-or-later
# License-Filename: LICENSE

import os
from contextlib import contextmanager
from functools import wraps


def none_on_exception(*exceptions):
    def _none_on_exception(fp):
        @wraps(fp)
        def wrapper(*args, **kwargs):
            try:
                return fp(*args, **kwargs)
            except exceptions:
                return None

        return wrapper

    return _none_on_exception


@contextmanager
def nullfd(fd):
    saveout = os.dup(fd)
    os.close(fd)
    os.open(os.devnull, os.O_RDWR)
    try:
        yield
    finally:
        os.dup2(saveout, fd)
        os.close(saveout)



#!/usr/bin/python3

import os
import sys
from pathlib import Path
from subprocess import run

WORKDIR = "/tmp"
work_path = Path(WORKDIR)

# TODO - temp
if Path("/usr/bin/topydo-helper").exists():
    sys.exit(0)

if "AUTOPKGTEST_ARTIFACTS" in os.environ:
    artifact_path = Path(os.environ["AUTOPKGTEST_ARTIFACTS"])
else:
    artifact_path = work_path

config_path = work_path / "topydo.conf"

start_path = Path(os.getcwd())
os.chdir(WORKDIR)

if config_path.exists():
    config_path.unlink()

with open(str(config_path), "w") as fp:
    fp.write("[topydo]\n")
    fp.write("filename = /tmp/todo.txt\n")

cp = run(["topydo", "--info"], capture_output=True, encoding="utf-8")
passed = False
for line in cp.stdout.split("\n"):
    if line == "task_path = /tmp/todo.txt":
        passed = True

with open(str(artifact_path / "todo.txt-helper-config.txt"), "w") as fp:
    fp.write(cp.stdout)

config_path.unlink()

os.chdir(str(start_path))

if not passed:
    print("Did not find task file path")

    sys.exit(1)




import configparser
import io
import subprocess
import textwrap

tconfig = None


class TDTBaseException(Exception):
    pass


class TConfig(object):
    def __init__(self, config_text, section='DEFAULT', defaults={}):
        self._section = section

        self._config = configparser.ConfigParser(defaults=defaults)
        try:
            conf_str = "[{}]\n".format(self._section) + config_text
            conf_fp = io.StringIO(conf_str)
            self._config.read_file(conf_fp)
        except FileNotFoundError:
            pass

    def get(self, tag):
        try:
            val = self._config.get(self._section, tag)
        except configparser.NoOptionError:
            raise TDTBaseException("Option not found: " + tag)

        val = val.strip(" \n\"'")

        return val


def get_var(tag):
    global tconfig

    if tconfig is None:
        cp = subprocess.run(
            ["todo.txt", "--info"],
            capture_output=True,
            encoding="utf-8"
        )
        
        if cp.stdout:
            tconfig = TConfig(cp.stdout)


    return tconfig.get(tag)




import textwrap
import subprocess
from unittest.mock import Mock

import pytest

from todo_txt_base import tconfig


@pytest.fixture
def config(monkeypatch):
    config_data = textwrap.dedent(
        """
            executable = /usr/bin/topydo
            config_file = /etc/topydo.conf
            extraneous = foo
        """
    )

    cpmock = Mock()
    cpmock.stdout = config_data
    runmock = Mock(return_value=cpmock)
    monkeypatch.setattr(tconfig.subprocess,"run", runmock)

    return None


def test_config_internal(config):
    assert tconfig.get_var("executable") == "/usr/bin/topydo"


def test_config_miss(config):
    with pytest.raises(tconfig.TDTBaseException):
        tconfig.get_var("bogus")




#!/usr/bin/python3

import datetime
import os
import sys

import configargparse


def parse_args():
    conf_paths = [
        "/etc/todo-txt-gtd/tdtbackup.conf",
        "~/.config/todo-txt-gtd/tdtbackup.conf",
        "~/.tdtbackup.conf",
        "~/.todo-txt/tdtbackup.conf",
    ]
    for confarg in ("-c", "--config-file"):
        if any(True for arg in sys.argv if arg.startswith(confarg)):
            conf_paths = []
            break

    parser = configargparse.ArgumentParser(
        default_config_files=conf_paths,
        description="Back up the todo.txt file.",
    )

    parser.add_argument(
        "-f",
        "--file",
        help="the todo.txt file location (defaults to Dropbox)",
        type=str,
        default="~/Dropbox/todo/todo.txt",
    )

    parser.add_argument(
        "-b",
        "--backupdir",
        help='the backup location (defaults to "todo/backup" in Dropbox)',
        type=str,
        default="~/Dropbox/todo/backup",
    )

    parser.add_argument(
        "-n",
        "--num",
        help="the number of backup files to keep (defaults to 14)",
        type=int,
        default=14,
    )

    parser.add_argument(
        "-c",
        "--config_file",
        is_config_file=True,
        help="alternate config file",
    )

    args = parser.parse_args()
    args.file = os.path.expanduser(args.file)
    args.backupdir = os.path.expanduser(args.backupdir)

    return args


def backup_path(backup_dir):
    datestr = str(datetime.datetime.now())

    for badstr in [" ", ":", "."]:
        datestr = datestr.replace(badstr, "-")

    return os.path.join(backup_dir, "todo.txt-{}".format(datestr))


def cleanup(backup_dir, num_files):
    files = sorted(os.listdir(backup_dir))

    if len(files) > num_files:
        for delfile in files[: len(files) - num_files]:
            os.remove(os.path.join(backup_dir, delfile))


def backup(backupdir, filename, num):
    if not os.path.exists(backupdir):
        os.makedirs(backupdir)

    # use read/write for a dos2unix action
    with open(filename, "r") as rp:
        with open(backup_path(backupdir), "w") as wp:
            wp.write(rp.read())

    cleanup(backupdir, num)

def backup_done(backupdir, todopath):
    src = os.path.join(os.path.dirname(todopath), "done.txt")
    dest = os.path.join(backupdir, "done.txt")

    try:
        with open(src, "r") as rp:
            with open(dest, "w") as wp:
                wp.write(rp.read())
    except FileNotFoundError:
        # this backup is best effort
        pass

def main():
    args = parse_args()
    backup(args.backupdir, args.file, args.num)
    backup_done(args.backupdir, args.file)


if __name__ == "__main__":
    main()



# Copyright (c) 2020 David Steele <dsteele@gmail.com>
#
# SPDX-License-Identifier: GPL-2.0-or-later
# License-Filename: LICENSE

import sys
from functools import wraps
from pathlib import Path
from subprocess import run

from todo_txt_base.tconfig import get_var, TDTBaseException

PRE_HOOK_DIR = "/etc/todo.txt-base/prehooks"
POST_HOOK_DIR = "/etc/todo.txt-base/posthooks"


def run_hooks(dir, task_path):
    dir_path = Path(dir)

    script_paths = [x for x in dir_path.iterdir() if x.is_file]

    scripts = sorted([str(x) for x in script_paths])

    for script in scripts:
        run([script, task_path])


def prehook(fn):
    @wraps(fn)
    def wrapper(exe_path, task_path, *args, **kwargs):
        run_hooks(PRE_HOOK_DIR, task_path)
        return fn(exe_path, task_path, *args, **kwargs)

    return wrapper


def posthook(fn):
    @wraps(fn)
    def wrapper(exe_path, task_path, *args, **kwargs):
        returnval = fn(exe_path, task_path, *args, **kwargs)
        run_hooks(POST_HOOK_DIR, task_path)
        return returnval

    return wrapper


def tdtwrapper(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        try:
            task_path = get_var("task_path")
            exe_path = get_var("executable")
            return fn(exe_path, task_path, *args, **kwargs)
        except TDTBaseException:
            sys.exit(1)

    return wrapper
